Imports System
Imports System.Threading.Tasks
Imports System.Windows.Forms
Imports System.Net
Imports System.Net.Sockets
Imports System.IO

' stand-alone, single-file version of
' the WinForm application for MSDN article
' "Asynchronous TCP Sockets as an Alternative to WCF"

Namespace DemoFormClient
  Friend NotInheritable Class Program
        <STAThread>
        Shared Sub Main()
            Application.EnableVisualStyles()
            Application.SetCompatibleTextRenderingDefault(False)
            Application.Run(New Form1)
        End Sub
  End Class

  Public Class Form1
	  Inherits Form
	Private Shared async Function SendRequest(ByVal server As String, ByVal port As Integer, ByVal method As String, ByVal data As String) As Task(Of String)
	  Try
		' set up IP address of server
		Dim ipAddress As IPAddress = Nothing
		Dim ipHostInfo As IPHostEntry = Dns.GetHostEntry(server)
		For i As Integer = 0 To ipHostInfo.AddressList.Length - 1
		  If ipHostInfo.AddressList(i).AddressFamily = AddressFamily.InterNetwork Then
			ipAddress = ipHostInfo.AddressList(i)
			Exit For
		  End If
		Next i
		If ipAddress Is Nothing Then
		  Throw New Exception("Unable to find an IPv4 address for server")
		End If

		Dim client As New TcpClient
		await client.ConnectAsync(ipAddress, port) ' connect to the server

                Dim networkStream As NetworkStream = client.GetStream
		Dim writer As New StreamWriter(networkStream)
		Dim reader As New StreamReader(networkStream)

		writer.AutoFlush = True
		Dim requestData As String = "method=" & method & "&" & "data=" & data & "&eor" ' 'end-of-requet'
		await writer.WriteLineAsync(requestData)
                Dim response As String = Await reader.ReadLineAsync

		client.Close()

		Return response

	  Catch ex As Exception
		Return ex.Message
	  End Try
	End Function ' SendRequest

	Private async Sub button1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles button1.Click
	  Try
		Dim server As String = "VTE.redmond.corp.microsoft.com"
		Dim port As Integer = 50000
		Dim method As String = CStr(comboBox1.SelectedItem)
		Dim data As String = textBox1.Text

'		technique 1, String Return w/ inline await String sResponse = await SendRequest(server, port, method, data)
'       Dim dResponse As Double = Double.Parse(sResponse)
                '       listBox1.Items.Add("Sent request, waiting for response")
                '       listBox1.Items.Add("Received response: " & dResponse.ToString("F2"))

                '       technique 2, Task(Of String) Return w/ later await

		Dim tsResponse As Task(Of String) = SendRequest(server, port, method, data)
		listBox1.Items.Add("Sent request, waiting for response")
                Dim dResponse As Double = Double.Parse(Await tsResponse)
		listBox1.Items.Add("Received response: " & dResponse.ToString("F2"))
	  Catch ex As Exception
		listBox1.Items.Add(ex.Message)
	  End Try
	End Sub

	Private Sub button2_Click(ByVal sender As Object, ByVal e As EventArgs) Handles button2.Click
	  listBox1.Items.Add("Hello")
	End Sub

	Private components As System.ComponentModel.IContainer = Nothing
	Private comboBox1 As System.Windows.Forms.ComboBox
	Private textBox1 As System.Windows.Forms.TextBox
	Private WithEvents button1 As System.Windows.Forms.Button
	Private listBox1 As System.Windows.Forms.ListBox
	Private WithEvents button2 As System.Windows.Forms.Button
	Private label1 As System.Windows.Forms.Label
	Private label2 As System.Windows.Forms.Label
	Private label3 As System.Windows.Forms.Label
	Private label4 As System.Windows.Forms.Label

	Public Sub New()
	  InitializeComponent()
	End Sub

	Protected Overrides Overloads Sub Dispose(ByVal disposing As Boolean)
	  If disposing AndAlso (components IsNot Nothing) Then
		components.Dispose()
	  End If
	  MyBase.Dispose(disposing)
	End Sub


	Private Sub InitializeComponent()
            Me.comboBox1 = New System.Windows.Forms.ComboBox
            Me.textBox1 = New System.Windows.Forms.TextBox
            Me.button1 = New System.Windows.Forms.Button
            Me.listBox1 = New System.Windows.Forms.ListBox
            Me.button2 = New System.Windows.Forms.Button
            Me.label1 = New System.Windows.Forms.Label
            Me.label2 = New System.Windows.Forms.Label
            Me.label3 = New System.Windows.Forms.Label
            Me.label4 = New System.Windows.Forms.Label
            Me.SuspendLayout()
	  ' 
	  ' comboBox1
	  ' 
	  Me.comboBox1.FormattingEnabled = True
            Me.comboBox1.Items.AddRange(New Object() {"average", "minimum"})
	  Me.comboBox1.Location = New System.Drawing.Point(14, 37)
	  Me.comboBox1.Name = "comboBox1"
	  Me.comboBox1.Size = New System.Drawing.Size(168, 21)
	  Me.comboBox1.TabIndex = 0
	  ' 
	  ' textBox1
	  ' 
	  Me.textBox1.Location = New System.Drawing.Point(14, 82)
	  Me.textBox1.Name = "textBox1"
	  Me.textBox1.Size = New System.Drawing.Size(433, 20)
	  Me.textBox1.TabIndex = 1
	  ' 
	  ' button1
	  ' 
	  Me.button1.Location = New System.Drawing.Point(14, 131)
	  Me.button1.Name = "button1"
	  Me.button1.Size = New System.Drawing.Size(75, 23)
	  Me.button1.TabIndex = 2
	  Me.button1.Text = "Send Async"
	  Me.button1.UseVisualStyleBackColor = True
            '	  AddHandler Me.button1.Click, AddressOf New System.EventHandler(Me.button1_Click)
	  ' 
	  ' listBox1
	  ' 
	  Me.listBox1.FormattingEnabled = True
	  Me.listBox1.Location = New System.Drawing.Point(12, 182)
	  Me.listBox1.Name = "listBox1"
	  Me.listBox1.Size = New System.Drawing.Size(435, 160)
	  Me.listBox1.TabIndex = 4
	  ' 
	  ' button2
	  ' 
	  Me.button2.Location = New System.Drawing.Point(107, 131)
	  Me.button2.Name = "button2"
	  Me.button2.Size = New System.Drawing.Size(75, 23)
	  Me.button2.TabIndex = 5
	  Me.button2.Text = "Say Hello"
	  Me.button2.UseVisualStyleBackColor = True
            '	  AddHandler Me.button2.Click, AddressOf New System.EventHandler(Me.button2_Click)
	  ' 
	  ' label1
	  ' 
	  Me.label1.AutoSize = True
	  Me.label1.Location = New System.Drawing.Point(13, 21)
	  Me.label1.Name = "label1"
	  Me.label1.Size = New System.Drawing.Size(75, 13)
	  Me.label1.TabIndex = 6
	  Me.label1.Text = "Select method"
	  ' 
	  ' label2
	  ' 
	  Me.label2.AutoSize = True
	  Me.label2.Location = New System.Drawing.Point(13, 66)
	  Me.label2.Name = "label2"
	  Me.label2.Size = New System.Drawing.Size(184, 13)
	  Me.label2.TabIndex = 7
	  Me.label2.Text = "Enter space-delimited data (like 3 1 8)"
	  ' 
	  ' label3
	  ' 
	  Me.label3.AutoSize = True
	  Me.label3.Location = New System.Drawing.Point(13, 115)
	  Me.label3.Name = "label3"
	  Me.label3.Size = New System.Drawing.Size(70, 13)
	  Me.label3.TabIndex = 8
	  Me.label3.Text = "Send request"
	  ' 
	  ' label4
	  ' 
	  Me.label4.AutoSize = True
	  Me.label4.Location = New System.Drawing.Point(188, 136)
	  Me.label4.Name = "label4"
	  Me.label4.Size = New System.Drawing.Size(259, 13)
	  Me.label4.TabIndex = 9
	  Me.label4.Text = "(responsive even while waiting for service to respond)"
	  ' 
	  ' Form1
	  ' 
	  Me.AutoScaleDimensions = New System.Drawing.SizeF(6F, 13F)
	  Me.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font
	  Me.ClientSize = New System.Drawing.Size(460, 362)
	  Me.Controls.Add(Me.label4)
	  Me.Controls.Add(Me.label3)
	  Me.Controls.Add(Me.label2)
	  Me.Controls.Add(Me.label1)
	  Me.Controls.Add(Me.button2)
	  Me.Controls.Add(Me.listBox1)
	  Me.Controls.Add(Me.button1)
	  Me.Controls.Add(Me.textBox1)
	  Me.Controls.Add(Me.comboBox1)
	  Me.Name = "Form1"
	  Me.Text = "WinForm TCP Service Client"
	  Me.ResumeLayout(False)
	  Me.PerformLayout()
	End Sub ' InitializeComponent()


  End Class ' Form1




End Namespace ' ns