﻿Imports System
Imports System.Net
Imports System.Net.Sockets
Imports System.IO
Imports System.Threading.Tasks

' TCP service that computes mean or average of an array

Namespace SocketServerTcpListener
  Friend Class ServiceProgram
	Shared Sub Main(ByVal args() As String)
	  Try
		Dim port As Integer = 50000
		Dim service As New AsyncService(port)
		service.Run() ' very specific service
		Console.ReadLine()
	  Catch ex As Exception
		Console.WriteLine(ex.Message)
		Console.ReadLine()
	  End Try
	End Sub
  End Class ' Program

  Public Class AsyncService
	Private ipAddress As IPAddress
	Private port As Integer

	Public Sub New(ByVal port As Integer)
	  ' set up port and determine IP Address
	  Me.port = port
            Dim hostName As String = Dns.GetHostName
	  Dim ipHostInfo As IPHostEntry = Dns.GetHostEntry(hostName)

	  'this.ipAddress = IPAddress.Any; // allows requests to any of server's  addresses

	  Me.ipAddress = Nothing
	  For i As Integer = 0 To ipHostInfo.AddressList.Length - 1
		If ipHostInfo.AddressList(i).AddressFamily = AddressFamily.InterNetwork Then
		  Me.ipAddress = ipHostInfo.AddressList(i)
		  Exit For
		End If
	  Next i
	  If Me.ipAddress Is Nothing Then
		Throw New Exception("No IPv4 address for server")
	  End If
	End Sub ' AsyncService ctor

	' normally the return would be Task but if so, we'd get a compiler warning in Main
	' for not using await, but if we prepend await that's an error because Main returns void.
	' A work-around attempt of making Main return a Task is not allowed. In short, void makes
	' sense in this situation.

	'public async Task Run()
	Public async Sub Run()
	  Dim listener As New TcpListener(Me.ipAddress, Me.port)
	  listener.Start()
	  Console.WriteLine(vbLf & "Array Min and Avg service is now running on port " & Me.port)
            Console.WriteLine("Hit <enter> to stop service" & vbCrLf)

	  Do
		Try
                    'Console.WriteLine("Waiting for a request ...")
                    Dim tcpClient As TcpClient = Await listener.AcceptTcpClientAsync
                    Dim t As Task = Process(tcpClient)
                Catch ex As Exception
                    Console.WriteLine(ex.Message)
		End Try
	  Loop
	End Sub ' Start

	Private async Function Process(ByVal tcpClient As TcpClient) As Task
            Dim clientEndPoint As String = tcpClient.Client.RemoteEndPoint.ToString
            'Console.WriteLine("Received connection request from " & clientEndPoint)
	  Console.WriteLine("Received connection request from 123.45.678.999")
	  Try
                Dim networkStream As NetworkStream = tcpClient.GetStream
		Dim reader As New StreamReader(networkStream)
		Dim writer As New StreamWriter(networkStream)
		writer.AutoFlush = True
		Do
		  Dim request As String = await reader.ReadLineAsync
		  If request IsNot Nothing Then
			Console.WriteLine("Received service request: " & request)
			Dim response As String = AsyncService.Response(request)
			Console.WriteLine("Computed response is: " & response & vbCrLf)
			await writer.WriteLineAsync(response)
		  Else
			Exit Do ' client closede connection
		  End If
		Loop
		tcpClient.Close()
	  Catch ex As Exception
		Console.WriteLine(ex.Message)
		If tcpClient.Connected Then
		  tcpClient.Close()
		End If
	  End Try
	End Function ' Process

	Private Shared Function Response(ByVal request As String) As String
	  ' assumes request has form like method=average&data=1.1 2.2 3.3&eor
	  ' eor stands for end-of-request
	  ' dummy delay bssed on the first numeric value
	  Dim pairs() As String = request.Split("&"c)
	  Dim methodName As String = pairs(0).Split("="c)(1)
	  Dim valueString As String = pairs(1).Split("="c)(1)

	  Dim values() As String = valueString.Split(" "c)
	  Dim vals(values.Length - 1) As Double
	  For i As Integer = 0 To values.Length - 1
		vals(i) = Double.Parse(values(i))
	  Next i

	  Dim response_Renamed As String = ""
	  If methodName = "average" Then
		  response_Renamed &= Average(vals)
	  ElseIf methodName = "minimum" Then
		  response_Renamed &= Minimum(vals)
	  Else
		  response_Renamed &= "BAD methodName: " & methodName
	  End If

	  Dim delay As Integer = (CInt(Fix(vals(0)))) * 1000 ' dummy delay
	  System.Threading.Thread.Sleep(delay)

	  Return response_Renamed
	End Function ' Response

	Private Shared Function Average(ByVal vals() As Double) As Double
	  Dim sum As Double = 0.0
	  For i As Integer = 0 To vals.Length - 1
		sum += vals(i)
	  Next i
	  Return sum / vals.Length
	End Function

	Private Shared Function Minimum(ByVal vals() As Double) As Double
	  Dim min As Double = vals(0)

	  For i As Integer = 0 To vals.Length - 1
		If vals(i) < min Then
			min = vals(i)
		End If
	  Next i
	  Return min
	End Function

  End Class ' AsyncServer

End Namespace ' ns
